@@ -1,8 +1,8 @@
-/*	$calcurse: apoint.h,v 1.4 2006/09/14 15:06:54 culot Exp $	*/
+/*	$calcurse: apoint.h,v 1.9 2007/08/15 15:33:54 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2006 Frederic Culot
+ * Copyright (c) 2004-2007 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,16 +27,22 @@
 #ifndef CALCURSE_APOINT_H
 #define CALCURSE_APOINT_H
 
-#include <ncurses.h>
 #include "notify.h"
+#include "recur.h"
+#include "vars.h"
 
 #define HRMIN_SIZE 6
 #define MESG_MAXSIZE 256
 
+#define APOINT_NULL		0x0
+#define APOINT_NOTIFY		0x1	/* Item needs to be notified */
+#define APOINT_NOTIFIED		0x2	/* Item was already notified */
+
 typedef struct apoint_llist_node {
 	struct apoint_llist_node *next;
 	long start;		/* seconds since 1 jan 1970 */
 	long dur;		/* duration of the appointment in seconds */
+	char state;		/* 8 bits to store item state */
 	char *mesg;
 } apoint_llist_node_t;
 
@@ -47,20 +53,27 @@ typedef struct apoint_llist {
 
 extern apoint_llist_t *alist_p;
 
-int apoint_llist_init(void);
-apoint_llist_node_t *apoint_new(char *, long, long);
-unsigned apoint_inday(apoint_llist_node_t *o, long start);
-void apoint_sec2str(apoint_llist_node_t *o, 
-	int type, long day, char *start, char *end);
-void apoint_write(apoint_llist_node_t *o, FILE * f);
-apoint_llist_node_t *apoint_scan(FILE * f, struct tm start, struct tm end);
-void apoint_delete_bynum(long start, unsigned num);
-void display_item_date(WINDOW *win, int color, apoint_llist_node_t *i,
-		int type, long date, int y, int x);
-int get_item_line(int item_nb, int nb_events_inday);
-void scroll_pad_down(int item_nb, int nb_events_inday, int win_length);
-void scroll_pad_up(int item_nb, int nb_events_inday);
-struct notify_app_s *apoint_check_next(struct notify_app_s *app, long start);
-apoint_llist_node_t *apoint_recur_s2apoint_s(recur_apoint_llist_node_t *p);
+int 			apoint_llist_init(void);
+void			apoint_hilt_set(int);
+void			apoint_hilt_decrease(void);
+void			apoint_hilt_increase(void);
+int			apoint_hilt(void);
+apoint_llist_node_t    *apoint_new(char *, long, long, char);
+void			apoint_add(void);
+void			apoint_delete(conf_t *, unsigned *, unsigned *);
+unsigned 		apoint_inday(apoint_llist_node_t *, long);
+void 			apoint_sec2str(apoint_llist_node_t *, int, long, 
+			    char *, char *);
+void 			apoint_write(apoint_llist_node_t *, FILE *);
+apoint_llist_node_t    *apoint_scan(FILE *, struct tm, struct tm, char);
+void 			apoint_delete_bynum(long, unsigned);
+void 			display_item_date(WINDOW *, int, apoint_llist_node_t *,
+			    int, long, int, int);
+void 			apoint_scroll_pad_down(int, int);
+void 			apoint_scroll_pad_up(int);
+struct notify_app_s    *apoint_check_next(struct notify_app_s *, long);
+apoint_llist_node_t    *apoint_recur_s2apoint_s(recur_apoint_llist_node_t *);
+void 			apoint_switch_notify(void);
+void			apoint_update_panel(window_t *, int);
 
 #endif /* CALCURSE_APOINT_H */
