@@ -1,8 +1,8 @@
-/*	$calcurse: args.h,v 1.2 2006/09/16 09:09:06 culot Exp $	*/
+/*	$calcurse: args.h,v 1.8 2007/07/28 13:11:42 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2006 Frederic Culot
+ * Copyright (c) 2004-2007 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,16 +27,6 @@
 #ifndef CALCURSE_ARGS_H
 #define CALCURSE_ARGS_H
 
-void usage();
-void usage_try();
-int parse_args(int argc, char **argv, int colr);
-void version_arg();
-void help_arg();
-void todo_arg(int colr);
-void next_arg(void);
-int app_arg(int add_line, int year, int month, int day, long date);
-void date_arg(char *ddate, int add_line);
-int check_date(char *date);
-void arg_print_date(long date);
+int 	parse_args(int, char **, conf_t *);
 
 #endif /* CALCURSE_ARGS_H */
