@@ -1,8 +1,8 @@
-/*	$calcurse: day.h,v 1.6 2006/09/14 15:09:10 culot Exp $	*/
+/*	$calcurse: day.h,v 1.15 2007/08/15 15:37:10 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2006 Frederic Culot
+ * Copyright (c) 2004-2007 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,8 +27,7 @@
 #ifndef CALCURSE_DAY_H
 #define CALCURSE_DAY_H
 
-#include "vars.h"
-#include "apoint.h"
+#include "calendar.h"
 
 #define MAX_TYPES	4
 
@@ -37,37 +36,37 @@
 #define RECUR_APPT	3
 #define APPT		4
 
+typedef struct {
+	unsigned nb_events;
+	unsigned nb_apoints; 
+} day_items_nb_t;
+
 struct day_item_s {
 	struct day_item_s *next;
-	int type;		/* (recursive or normal) event or appointment */
 	long start;		/* seconds since 1 jan 1970 */
-	char *mesg;		/* item description */
-	int evnt_id;		/* event identifier */
 	long appt_dur;		/* appointment duration in seconds */
+	int type;		/* (recursive or normal) event or appointment */
+	int evnt_id;		/* event identifier */
 	int appt_pos;		/* real position in recurrent list */
+	char state;		/* appointment state */
+	char *mesg;		/* item description */
 };
 
 struct day_saved_item_s {
-	int type ;
+	char start[BUFSIZ]; 
+	char end[BUFSIZ];
+	char state;
+	char type ;
 	char *mesg;
-	char start[MAX_LENGTH]; 
-	char end[MAX_LENGTH];
 };
 
-int day_store_items(long date, int *pnb_events, int *pnb_apoints);
-void day_free_list(void);
-int day_store_recur_events(long date);
-int day_store_events(long date);
-int day_store_recur_apoints(long date);
-int day_store_apoints(long date);
-struct day_item_s *day_add_event(int type, char *mesg, long day, int id);
-struct day_item_s *day_add_apoint(int type, char *mesg, long start, long dur,
-				  int real_pos);
-void day_write_pad(long date, int width, int length, int incolor, int colr);
-apoint_llist_node_t *day_item_s2apoint_s(struct day_item_s *p);
-void day_popup_item(void);
-int day_check_if_item(int year, int month, int day);
-int day_erase_item(long date, int item_number);
-struct day_item_s *day_get_item(int item_number);
+day_items_nb_t 	       *day_process_storage(date_t *, bool, day_items_nb_t *);
+void 			day_write_pad(long, int, int, int);
+void 			day_popup_item(void);
+int	 		day_check_if_item(date_t);
+void 			day_edit_item(void);
+int 			day_erase_item(long, int, int);
+struct day_item_s      *day_get_item(int);
+int 			day_item_nb(long, int, int);
 
 #endif /* CALCURSE_DAY_H */
