@@ -1,8 +1,8 @@
-/*	$calcurse: recur.c,v 1.13 2006/09/16 15:23:05 culot Exp $	*/
+/*	$calcurse: recur.c,v 1.30 2007/10/21 13:42:34 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2006 Frederic Culot
+ * Copyright (c) 2004-2007 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
  *
  */
 
-#include <ncurses.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -33,13 +32,9 @@
 
 #include "i18n.h"
 #include "utils.h"
-#include "apoint.h"
-#include "event.h"
-#include "recur.h"
 #include "notify.h"
-#include "args.h"
 #include "day.h"
-#include "vars.h"
+#include "recur.h"
 
 recur_apoint_llist_t *recur_alist_p;
 struct recur_event_s *recur_elist;
@@ -55,8 +50,9 @@ int recur_apoint_llist_init(void)
 }
 
 /* Insert a new recursive appointment in the general linked list */
-recur_apoint_llist_node_t *recur_apoint_new(char *mesg, long start, long dur, 
-	int type, int freq, long until, struct days_s *except)
+recur_apoint_llist_node_t *
+recur_apoint_new(char *mesg, long start, long dur, char state, int type, 
+    int freq, long until, struct days_s *except)
 {
 	recur_apoint_llist_node_t *o, **i;
 	o = (recur_apoint_llist_node_t *) 
@@ -64,8 +60,9 @@ recur_apoint_llist_node_t *recur_apoint_new(char *mesg, long start, long dur,
 	o->rpt = (struct rpt_s *) malloc(sizeof(struct rpt_s));
 	o->mesg = (char *) malloc(strlen(mesg) + 1);
 	o->exc = (struct days_s *) malloc(sizeof(struct days_s));
-	strcpy(o->mesg, mesg);
+	strncpy(o->mesg, mesg, strlen(mesg) + 1);
 	o->start = start;
+	o->state = state;
 	o->dur = dur;
 	o->rpt->type = type;
 	o->rpt->freq = freq;
@@ -96,7 +93,7 @@ struct recur_event_s *recur_event_new(char *mesg, long day, int id,
 	o->rpt = (struct rpt_s *) malloc(sizeof(struct rpt_s));
 	o->mesg = (char *) malloc(strlen(mesg) + 1);
 	o->exc = (struct days_s *) malloc(sizeof(struct days_s));
-	strcpy(o->mesg, mesg);
+	strncpy(o->mesg, mesg, strlen(mesg) + 1);
 	o->day = day;
 	o->id = id;
 	o->rpt->type = type;
@@ -119,24 +116,31 @@ struct recur_event_s *recur_event_new(char *mesg, long day, int id,
  * Correspondance between the defines on recursive type, 
  * and the letter to be written in file. 
  */
-char recur_def2char(int define){
+char 
+recur_def2char(recur_types_t define)
+{
 	char recur_char;
+	char *error = _("FATAL ERROR in recur_def2char: unknown recur type\n");
 
 	switch (define) {
-	case 1:
+	case RECUR_DAILY:
 		recur_char = 'D';
 		break;	
-	case 2:
+	case RECUR_WEEKLY:
 		recur_char = 'W';
 		break;
-	case 3:
+	case RECUR_MONTHLY:
 		recur_char = 'M';
 		break;
-	case 4:
+	case RECUR_YEARLY:
 		recur_char = 'Y';
 		break;
+	default:
+		fputs(error, stderr);
+		exit(EXIT_FAILURE);
 	}
-	return recur_char;
+
+	return (recur_char);
 }
 
 /* 
@@ -145,6 +149,7 @@ char recur_def2char(int define){
  */
 int recur_char2def(char type){
 	int recur_def;
+	char *error = _("FATAL ERROR in recur_char2def: unknown char\n");
 
 	switch (type) {
 	case 'D':
@@ -159,12 +164,16 @@ int recur_char2def(char type){
 	case 'Y':
 		recur_def = RECUR_YEARLY;
 		break;
+	default:
+		fputs(error, stderr);
+		exit(EXIT_FAILURE);
 	}
-	return recur_def;
+	return (recur_def);
 }
 
 /* Write days for which recurrent items should not be repeated. */
-void recur_write_exc(struct days_s *exc, FILE *f) {
+static void 
+recur_write_exc(struct days_s *exc, FILE *f) {
 	struct tm *lt;
 	time_t t;
 	int st_mon, st_day, st_year;
@@ -180,77 +189,10 @@ void recur_write_exc(struct days_s *exc, FILE *f) {
 	}
 }
 
-/* Writting of a recursive appointment into file. */
-void recur_apoint_write(recur_apoint_llist_node_t *o, FILE *f)
-{
-	struct tm *lt;
-	time_t t;
-
-	t = o->start;
-	lt = localtime(&t);
-	fprintf(f, "%02u/%02u/%04u @ %02u:%02u",
-		lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year,
-		lt->tm_hour, lt->tm_min);
-
-	t = o->start + o->dur;
-	lt = localtime(&t);
-	fprintf(f, " -> %02u/%02u/%04u @ %02u:%02u",
-		lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year,
-		lt->tm_hour, lt->tm_min);
-
-	t = o->rpt->until;
-	if (t == 0) { /* We have an endless recurrent appointment. */
-		fprintf(f, " {%d%c", o->rpt->freq, 
-			recur_def2char(o->rpt->type)); 
-		if (o->exc != 0) recur_write_exc(o->exc, f);
-		fprintf(f, "} |%s\n", o->mesg);
-	} else {
-		lt = localtime(&t);
-		fprintf(f, " {%d%c -> %02u/%02u/%04u",
-			o->rpt->freq, recur_def2char(o->rpt->type),
-			lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year);
-		if (o->exc != 0) recur_write_exc(o->exc, f);
-		fprintf(f,"} |%s\n", o->mesg);
-	}
-}
-
-/* Writting of a recursive event into file. */
-void recur_event_write(struct recur_event_s *o, FILE *f)
-{
-	struct tm *lt;
-	time_t t;
-	int st_mon, st_day, st_year;
-	int end_mon, end_day, end_year;
-	
-	t = o->day;
-	lt = localtime(&t);
-	st_mon = lt->tm_mon + 1;
-	st_day = lt->tm_mday;
-	st_year = lt->tm_year + 1900;
-	t = o->rpt->until;
-	if (t == 0) { /* We have an endless recurrent event. */
-		fprintf(f, "%02u/%02u/%04u [%d] {%d%c",
-			st_mon, st_day, st_year, o->id, o->rpt->freq,
-			recur_def2char(o->rpt->type));
-		if (o->exc != 0) recur_write_exc(o->exc, f);
-		fprintf(f,"} %s\n", o->mesg);
-	} else {
-		lt = localtime(&t);
-		end_mon = lt->tm_mon + 1;
-		end_day = lt->tm_mday;
-		end_year = lt->tm_year + 1900;
-		fprintf(f, "%02u/%02u/%04u [%d] {%d%c -> %02u/%02u/%04u",
-			st_mon, st_day, st_year, o->id, 
-			o->rpt->freq, recur_def2char(o->rpt->type),
-			end_mon, end_day, end_year);
-		if (o->exc != 0) recur_write_exc(o->exc, f);
-		fprintf(f, "} %s\n", o->mesg);
-	}		
-}
-
 /* Load the recursive appointment description */
-recur_apoint_llist_node_t *recur_apoint_scan(FILE * f, struct tm start, 
-	struct tm end, char type, int freq, struct tm until, struct days_s *exc)
+recur_apoint_llist_node_t *
+recur_apoint_scan(FILE * f, struct tm start, struct tm end, char type, 
+    int freq, struct tm until, struct days_s *exc, char state)
 {
 	struct tm *lt;
 	char buf[MESG_MAXSIZE], *nl;
@@ -293,7 +235,7 @@ recur_apoint_llist_node_t *recur_apoint_scan(FILE * f, struct tm start,
 	}
       
 	return recur_apoint_new(buf, tstart, tend - tstart, 
-		recur_char2def(type), freq, tuntil, exc);
+		state, recur_char2def(type), freq, tuntil, exc);
 }
 
 /* Load the recursive events from file */
@@ -337,6 +279,79 @@ struct recur_event_s *recur_event_scan(FILE * f, struct tm start, int id,
 		freq, tuntil, exc);
 }
 
+/* Writting of a recursive appointment into file. */
+static void 
+recur_apoint_write(recur_apoint_llist_node_t *o, FILE *f)
+{
+	struct tm *lt;
+	time_t t;
+
+	t = o->start;
+	lt = localtime(&t);
+	fprintf(f, "%02u/%02u/%04u @ %02u:%02u",
+	    lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year,
+	    lt->tm_hour, lt->tm_min);
+
+	t = o->start + o->dur;
+	lt = localtime(&t);
+	fprintf(f, " -> %02u/%02u/%04u @ %02u:%02u",
+	    lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year,
+	    lt->tm_hour, lt->tm_min);
+
+	t = o->rpt->until;
+	if (t == 0) { /* We have an endless recurrent appointment. */
+		fprintf(f, " {%d%c", o->rpt->freq, 
+		    recur_def2char(o->rpt->type)); 
+	} else {
+		lt = localtime(&t);
+		fprintf(f, " {%d%c -> %02u/%02u/%04u",
+		    o->rpt->freq, recur_def2char(o->rpt->type),
+		    lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year);
+	}
+	if (o->exc != 0) 
+		recur_write_exc(o->exc, f);
+	if (o->state & APOINT_NOTIFY)
+		fprintf(f, "} !");
+	else
+		fprintf(f, "} |");
+	fprintf(f, "%s\n", o->mesg);
+}
+
+/* Writting of a recursive event into file. */
+static void 
+recur_event_write(struct recur_event_s *o, FILE *f)
+{
+	struct tm *lt;
+	time_t t;
+	int st_mon, st_day, st_year;
+	int end_mon, end_day, end_year;
+	
+	t = o->day;
+	lt = localtime(&t);
+	st_mon = lt->tm_mon + 1;
+	st_day = lt->tm_mday;
+	st_year = lt->tm_year + 1900;
+	t = o->rpt->until;
+	if (t == 0) { /* We have an endless recurrent event. */
+		fprintf(f, "%02u/%02u/%04u [%d] {%d%c",
+			st_mon, st_day, st_year, o->id, o->rpt->freq,
+			recur_def2char(o->rpt->type));
+		if (o->exc != 0) recur_write_exc(o->exc, f);
+		fprintf(f,"} %s\n", o->mesg);
+	} else {
+		lt = localtime(&t);
+		end_mon = lt->tm_mon + 1;
+		end_day = lt->tm_mday;
+		end_year = lt->tm_year + 1900;
+		fprintf(f, "%02u/%02u/%04u [%d] {%d%c -> %02u/%02u/%04u",
+			st_mon, st_day, st_year, o->id, 
+			o->rpt->freq, recur_def2char(o->rpt->type),
+			end_mon, end_day, end_year);
+		if (o->exc != 0) recur_write_exc(o->exc, f);
+		fprintf(f, "} %s\n", o->mesg);
+	}		
+}
+
 /* Write recursive items to file. */
 void recur_save_data(FILE *f)
 {
@@ -352,47 +367,93 @@ void recur_save_data(FILE *f)
 	pthread_mutex_unlock(&(recur_alist_p->mutex));
 }
 
-/* Check if the recurrent item belongs to the selected day. */
-unsigned recur_item_inday(long item_start, struct days_s *item_exc,
-				int rpt_type, int rpt_freq,
-			  	long rpt_until, long day_start)
+/* 
+ * Check if the recurrent item belongs to the selected day,
+ * and if yes, return the real start time.
+ * This function was improved thanks to Tony's patch.
+ */
+unsigned 
+recur_item_inday(long item_start, struct days_s *item_exc,
+    int rpt_type, int rpt_freq, long rpt_until, long day_start)
 {
-	long day_end = day_start + DAYINSEC;
-	int inday = 0;
+#define YEARSTODAYS(x)	((x) * 365L + (x) / 4 - (x) / 100 + (x) / 400) 	
+
+	date_t start_date;
+	long day_end;
 	struct tm *lt;
+	int day_month, day_year, day_yday, day_mday, diff;
 	struct days_s *exc;
 	time_t t;
 	char *error = 
 		_("FATAL ERROR in recur_item_inday: unknown item type\n");
 
+	day_end = day_start + DAYINSEC;
+	t = day_start;
+	lt = localtime(&t);
+	day_month = lt->tm_mon;
+	day_year = lt->tm_year;
+	day_yday = lt->tm_yday;
+	day_mday = lt->tm_mday;
+	
 	for (exc = item_exc; exc != 0; exc = exc->next)
 		if (exc->st < day_end && exc->st >= day_start) 
 			return 0;
+
 	if (rpt_until == 0) /* we have an endless recurrent item */
 		rpt_until = day_end;
-	while (item_start <= day_end && item_start < rpt_until) {
-		if (item_start < day_end && item_start >= day_start) {
-			inday = item_start;
+
+	if (item_start > day_end || rpt_until < day_start) 
+		return 0;
+
+	t = item_start;
+	lt = localtime(&t);
+
+	switch (rpt_type) {
+		case RECUR_DAILY:
+			diff = (YEARSTODAYS(lt->tm_year) + lt->tm_yday) -
+			    (YEARSTODAYS(day_year) + day_yday);
+			if (diff % rpt_freq != 0)
+				return 0;
+			lt->tm_mday = day_mday;
+			lt->tm_mon = day_month;
+			lt->tm_year = day_year;
 			break;
-		}
-		t = item_start;
-		lt = localtime(&t);
-		if (rpt_type == RECUR_DAILY) {
-			lt->tm_mday += rpt_freq;
-		} else if (rpt_type == RECUR_WEEKLY) {
-			lt->tm_mday += rpt_freq * 7;
-		} else if (rpt_type == RECUR_MONTHLY) {
-			lt->tm_mon += rpt_freq;
-		} else if (rpt_type == RECUR_YEARLY) {
-			lt->tm_year += rpt_freq;
-		} else { /* NOT REACHED */
+		case RECUR_WEEKLY:
+			diff = (YEARSTODAYS(lt->tm_year) + lt->tm_yday) -
+			    (YEARSTODAYS(day_year) + day_yday);
+			if (diff % (rpt_freq*7) != 0)
+				return 0;
+			lt->tm_mday = day_mday;
+			lt->tm_mon = day_month;
+			lt->tm_year = day_year;
+			break;
+		case RECUR_MONTHLY:
+			diff = ((day_year - lt->tm_year) * 12) +
+			    (day_month - lt->tm_mon);
+			if (diff % rpt_freq != 0)
+				return 0;
+			lt->tm_mon = day_month;
+			lt->tm_year = day_year;
+			break;
+		case RECUR_YEARLY:
+			diff = day_year - lt->tm_year;
+			if (diff % rpt_freq != 0)
+				return 0;
+			lt->tm_year = day_year;
+			break;
+		default:
 			fputs(error, stderr);
 			exit(EXIT_FAILURE);
-		}
-		item_start = date2sec(lt->tm_year + 1900, lt->tm_mon + 1, 
-			lt->tm_mday, lt->tm_hour, lt->tm_min);
-	}	
-	return inday;
+	}
+	start_date.dd = lt->tm_mday;
+	start_date.mm = lt->tm_mon + 1;
+	start_date.yyyy = lt->tm_year + 1900;
+	item_start = date2sec(start_date, lt->tm_hour, lt->tm_min);
+
+	if (item_start < day_end && item_start >= day_start)
+		return item_start;
+	else
+		return 0;
 }
 
 /* 
@@ -510,12 +571,16 @@ void recur_apoint_erase(long start, unsigned num, unsigned delete_whole)
  *	o repetition end date
  * and then delete the selected item to recreate it as a recurrent one
  */
-void recur_repeat_item(int sel_year, int sel_month, int sel_day, 
-    int item_nb, int colr) {
+void 
+recur_repeat_item(void) 
+{
+	struct tm *lt;
+	time_t t;
 	int ch = 0;
 	int valid_date = 0, date_entered = 0;
 	int year = 0, month = 0, day = 0;
-	char user_input[MAX_LENGTH];
+	date_t until_date;
+	char user_input[BUFSIZ] = "";
 	char *mesg_type_1 = 
 	_("Enter the repetition type: (D)aily, (W)eekly, (M)onthly, (Y)early");
 	char *mesg_type_2 = _("[D/W/M/Y] ");
@@ -533,22 +598,24 @@ void recur_repeat_item(int sel_year, int sel_month, int sel_day,
 	char *mesg_older   = 
 	_("Sorry, the date you entered is older than the item start time.");
 	int type = 0, freq = 0;
+	int item_nb;
 	struct day_item_s *p; 
 	recur_apoint_llist_node_t *ra;
 	struct recur_event_s *re;
 	long until, date;
 
+	item_nb = apoint_hilt();
 	p = day_get_item(item_nb);
 	if (p->type != APPT && p->type != EVNT) {
 		status_mesg(wrong_type_1, wrong_type_2);
-		ch = wgetch(swin);
+		ch = wgetch(win[STA].p);
 		return;
 	}
 
 	while ( (ch != 'D') && (ch != 'W') && (ch != 'M') 
 	    && (ch != 'Y') && (ch != ESCAPE) ) {
 		status_mesg(mesg_type_1, mesg_type_2);
-		ch = wgetch(swin);	 		
+		ch = wgetch(win[STA].p);	 		
 		ch = toupper(ch);
 	}
 	if (ch == ESCAPE) {
@@ -560,23 +627,22 @@ void recur_repeat_item(int sel_year, int sel_month, int sel_day,
 
 	while (freq == 0) {
 		status_mesg(mesg_freq_1, "");
-		getstring(swin, colr, user_input, 0, 1);
-		if (strlen(user_input) != 0) {
+		if (getstring(win[STA].p, user_input, BUFSIZ, 0, 1) == 
+		    GETSTRING_VALID) {
 			freq = atoi(user_input);
 			if (freq == 0) {
 				status_mesg(mesg_wrong_freq, wrong_type_2);
-				wgetch(swin);
+				wgetch(win[STA].p);
 			}
-			strcpy(user_input, "");
-		} else {
+			user_input[0] = '\0';
+		} else
 			return;
-		}
 	}
 
 	while (!date_entered) {
 		status_mesg(mesg_until_1, "");
-		getstring(swin, colr, user_input, 0, 1);
-		if (strlen(user_input) != 0) {
+		if (getstring(win[STA].p, user_input, 11, 0, 1) == 
+		    GETSTRING_VALID) {
 			if (strlen(user_input) == 1 && 
 			    strncmp(user_input, "0", 1) == 0 )  {
 				until = 0;
@@ -586,11 +652,16 @@ void recur_repeat_item(int sel_year, int sel_month, int sel_day,
 				if (valid_date) {
 					sscanf(user_input, "%d / %d / %d", 
 						&month, &day, &year);	
-					until = date2sec(year, month, day, 0, 0);
+					t = p->start; lt = localtime(&t);
+					until_date.dd = day;
+					until_date.mm = month;
+					until_date.yyyy = year;
+					until = date2sec(until_date,
+					    lt->tm_hour, lt->tm_min);
 					if (until < p->start) {
 						status_mesg(mesg_older,
 							wrong_type_2);
-						wgetch(swin);
+						wgetch(win[STA].p);
 						date_entered = 0;
 					} else {
 						date_entered = 1;
@@ -600,18 +671,17 @@ void recur_repeat_item(int sel_year, int sel_month, int sel_day,
 					date_entered = 0;
 				}
 			}
-		} else {
+		} else
 			return;
-		}
 	}
 	
-	date = date2sec(sel_year, sel_month, sel_day, 0, 0);
-	day_erase_item(date, item_nb);
+	date = calendar_get_slctd_day_sec();
+	day_erase_item(date, item_nb, 0);
 	if (p->type == EVNT) {
 		re = recur_event_new(p->mesg, p->start, p->evnt_id, 
 			type, freq, until, NULL);
 	} else if (p->type == APPT) {
-		ra = recur_apoint_new(p->mesg, p->start, p->appt_dur, 
+		ra = recur_apoint_new(p->mesg, p->start, p->appt_dur, p->state,
 			type, freq, until, NULL);
 		if (notify_bar()) notify_check_repeated(ra);
 	} else { /* NOTREACHED */
@@ -660,8 +730,8 @@ struct days_s *recur_exc_scan(FILE *data_file)
  * Look in the appointment list if we have an item which starts before the item
  * stored in the notify_app structure (which is the next item to be notified).
  */
-struct notify_app_s *recur_apoint_check_next(
-	struct notify_app_s *app, long start, long day)
+struct notify_app_s *
+recur_apoint_check_next(struct notify_app_s *app, long start, long day)
 {
 	recur_apoint_llist_node_t *i;
 	long real_recur_start_time;
@@ -678,6 +748,7 @@ struct notify_app_s *recur_apoint_check_next(
 			if (real_recur_start_time > start) {
 				app->time = real_recur_start_time;	
 				app->txt = mycpy(i->mesg);
+				app->state = i->state;
 				app->got_app = 1;
 			} 
 		}
@@ -686,3 +757,80 @@ struct notify_app_s *recur_apoint_check_next(
 
 	return app;
 }
+
+/* Returns a structure containing the selected recurrent appointment. */
+recur_apoint_llist_node_t *recur_get_apoint(long date, int num)
+{
+	recur_apoint_llist_node_t *o;
+	int n = 0;
+	
+	pthread_mutex_lock(&(recur_alist_p->mutex));
+	for (o = recur_alist_p->root; o != 0; o = o->next) {
+		if (recur_item_inday(o->start, o->exc, o->rpt->type,
+			o->rpt->freq, o->rpt->until, date)) {
+			if (n == num) {
+				pthread_mutex_unlock(&(recur_alist_p->mutex));
+				return o;	
+			}	
+			n++;
+		}
+	}
+	/* NOTREACHED */
+	fputs(_("FATAL ERROR in recur_get_apoint: no such item\n"), stderr);
+	exit(EXIT_FAILURE);
+}
+
+/* Returns a structure containing the selected recurrent event. */
+struct recur_event_s *recur_get_event(long date, int num)
+{
+	struct recur_event_s *o;
+	int n = 0;
+	
+	for (o = recur_elist; o != 0; o = o->next) {
+		if (recur_item_inday(o->day, o->exc, o->rpt->type,
+			o->rpt->freq, o->rpt->until, date)) {
+			if (n == num) {
+				return o;	
+			}	
+			n++;
+		}
+	}
+	/* NOTREACHED */
+	fputs(_("FATAL ERROR in recur_get_event: no such item\n"), stderr);
+	exit(EXIT_FAILURE);
+}
+
+/* Switch recurrent item notification state. */
+void
+recur_apoint_switch_notify(long date, int recur_nb)
+{
+	int n, need_chk_notify;
+	recur_apoint_llist_node_t *o;
+
+	n = 0;
+	need_chk_notify = 0;
+
+	pthread_mutex_lock(&(recur_alist_p->mutex));
+	for (o = recur_alist_p->root; o != 0; o = o->next) {
+		if (recur_item_inday(o->start, o->exc, o->rpt->type,
+			o->rpt->freq, o->rpt->until, date)) {
+			if (n == recur_nb) {
+				o->state ^= APOINT_NOTIFY;	
+
+				if (notify_bar())
+					notify_check_repeated(o);
+
+				pthread_mutex_unlock(&(recur_alist_p->mutex));
+				if (need_chk_notify)
+					notify_check_next_app();
+				return;
+			}	
+			n++;
+		}
+	}
+	/* NOTREACHED */
+	fputs(_("FATAL ERROR in recur_apoint_switch_notify: no such item\n"), 
+	    stderr);
+	exit(EXIT_FAILURE);
+	
+}
