@@ -1,8 +1,8 @@
-/*	$calcurse: todo.h,v 1.3 2006/08/31 18:49:05 culot Exp $	*/
+/*	$calcurse: todo.h,v 1.9 2007/08/15 15:35:25 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2006 Frederic Culot
+ * Copyright (c) 2004-2007 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,8 @@
 #ifndef CALCURSE_TODO_H
 #define CALCURSE_TODO_H
 
+#include "wins.h"
+
 struct todo_s {
 	struct todo_s *next;
 	char *mesg;
@@ -35,11 +37,22 @@ struct todo_s {
 
 extern struct todo_s *todolist;
 
-int todo_new_item(int total, int colr); 
-struct todo_s *todo_add(char *mesg, int id);
-void todo_delete_bynum(unsigned num);
-struct todo_s *todo_get_item(int item_number);
-int todo_get_position(struct todo_s *i);
-int todo_chg_priority(int action, int item_num);
+void		todo_hilt_set(int);
+void		todo_hilt_decrease(void);
+void		todo_hilt_increase(void);
+int		todo_hilt(void);
+int		todo_nb(void);
+void		todo_set_nb(int);
+void		todo_set_first(int);
+void		todo_first_increase(void);
+void		todo_first_decrease(void);
+int		todo_hilt_pos(void);
+char	       *todo_saved_mesg(void);
+void 		todo_new_item(void); 
+struct todo_s  *todo_add(char *, int);
+void		todo_delete(conf_t *);
+void 		todo_chg_priority(int);
+void 		todo_edit_item(void);
+void		todo_update_panel(window_t *, int);
 
 #endif /* CALCURSE_TODO_H */
