@@ -1,4 +1,4 @@
-/*	$calcurse: vars.c,v 1.2 2006/09/15 15:38:14 culot Exp $	*/
+/*	$calcurse: vars.c,v 1.5 2007/10/21 13:40:13 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -24,7 +24,12 @@
  *
  */
 
+#include <stdlib.h>
+
 #include "i18n.h"
+#include "calendar.h"
+#include "custom.h"
+#include "wins.h"
 #include "vars.h"
 
 /*
@@ -72,17 +77,37 @@ char path_todo[] = "";
 char path_apts[] = "";
 char path_conf[] = "";
 
-/* 
- * Variables to handle calcurse windows: 
- * cwin = calendar window
- * awin = appointment window
- * twin = todo window
- * swin = status bar window
- */
-WINDOW *awin = NULL, *cwin = NULL, *twin = NULL, *swin = NULL;
-
 /* Variable to handle pads. */
 struct pad_s *apad;
 
 /* Variable to store notify-bar settings. */
 struct nbar_s *nbar;
+
+
+/*
+ * Variables init 
+ */
+void 
+vars_init(conf_t *conf)
+{
+	/* Variables for user configuration */
+	conf->confirm_quit = true; 
+	conf->confirm_delete = true; 
+	conf->auto_save = true;
+	conf->skip_system_dialogs = false;
+	conf->skip_progress_bar = false;
+	wins_set_layout(1);
+
+	calendar_set_first_day_of_week(MONDAY);
+
+	/* Pad structure to scroll text inside the appointment panel */
+	apad = (struct pad_s *) malloc(sizeof(struct pad_s));
+	apad->length = 1;
+	apad->first_onscreen = 0;
+
+	/* Attribute definitions for color and non-color terminals */
+	custom_init_attr();
+	
+	/* Start at the current date */
+	calendar_init_slctd_day();
+}
