@@ -1,8 +1,8 @@
-/*	$calcurse: vars.h,v 1.4 2006/09/15 15:37:45 culot Exp $	*/
+/*	$calcurse: vars.h,v 1.17 2007/10/21 13:40:13 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2006 Frederic Culot
+ * Copyright (c) 2004-2007 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,11 +36,8 @@
 #define APTS_PATH	".calcurse/apts"
 #define CONF_PATH	".calcurse/conf"
 
-#define MAX_LENGTH	512
-
 #define CTRL(x)         ((x) & 0x1f)
 #define ESCAPE		27
-
 #define ATTR_FALSE	0
 #define ATTR_TRUE	1
 #define ATTR_LOWEST	2
@@ -50,6 +47,10 @@
 #define ATTR_HIGHEST	6
 
 #define DAYINSEC  	86400 
+#define HOURINSEC	3600
+#define MININSEC	60
+
+#define STATUSHEIGHT	2
 
 struct pad_s {
 	int width;
@@ -59,24 +60,37 @@ struct pad_s {
 };
 
 struct nbar_s {		
-	int show;		/* display or hide the notify-bar */
-	int cntdwn;		/* warn when time left before next app < cntdwn */  
-	char *datefmt;		/* format for displaying date */
-	char *timefmt;		/* format for displaying time */
+	int show;			/* display or hide the notify-bar */
+	int cntdwn;			/* warn when time left before next app
+					 * becomes lesser than cntdwn */  
+	char datefmt[BUFSIZ];		/* format for displaying date */
+	char timefmt[BUFSIZ];		/* format for displaying time */
+	char cmd[BUFSIZ];		/* notification command */
+	char *shell;			/* user shell to launch notif. cmd */
 	pthread_mutex_t mutex;
 };
 
-extern int col, row;
-extern bool colorize;
-extern int days[12];
-extern char *monthnames[12];
-extern char *daynames[8];
-extern char path_dir[MAX_LENGTH];
-extern char path_todo[MAX_LENGTH];
-extern char path_apts[MAX_LENGTH];
-extern char path_conf[MAX_LENGTH];
-extern WINDOW *awin, *cwin, *twin, *swin;
-extern struct pad_s *apad;
-extern struct nbar_s *nbar;
+/* General configuration variables */
+typedef struct {
+	bool auto_save;
+	bool confirm_quit;
+	bool confirm_delete;
+	bool skip_system_dialogs;
+	bool skip_progress_bar;
+} conf_t;
+
+extern int 		col, row;
+extern bool 		colorize;
+extern int 		days[12];
+extern char            *monthnames[12];
+extern char            *daynames[8];
+extern char 		path_dir[BUFSIZ];
+extern char 		path_todo[BUFSIZ];
+extern char 		path_apts[BUFSIZ];
+extern char 		path_conf[BUFSIZ];
+extern struct pad_s    *apad;
+extern struct nbar_s   *nbar;
+
+void			vars_init(conf_t *conf);
 
 #endif /* CALCURSE_VARS_H */
